﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Windows.Browser;
using System.Xml.Linq;

namespace FeedReader.Models
{
    public class FeedModel : INotifyPropertyChanged
    {

        #region Static Members

        private const String URI = "http://www.wolter.biz/external/get_feed.php?forumsid={0}";
        // private const String URI = "../get_feed.php?forumsid={0}";
        private const String SHOW_THREAD_URI = "{0}showthread.php?t={1}";
        private const String SHOW_FORUM_URI = "{0}forumdisplay.php?f={1}";
        private const String JOBFORUMS_ID = "32";

        #endregion

        #region Private Fields

        private String ForumsURL;

        #endregion

        #region Public Properties

        private Boolean _initialized;
        public Boolean Initialized
        {
            get { return _initialized; }
            set
            {
                _initialized = value;
                //Call NotifyPropertyChanged when the source property is updated
                NotifyPropertyChanged("Initialized");
            }
        }

        private IEnumerable<Thread> _threadList;
        public IEnumerable<Thread> ThreadList
        {
            get { return _threadList; }
            set
            {
                _threadList = value;
                //Call NotifyPropertyChanged when the source property is updated
                NotifyPropertyChanged("ThreadList");
            }
        }

        #endregion  

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Public Methods

        public void ShowForum()
        {
            HtmlPage.Window.Navigate(new Uri(String.Format(SHOW_FORUM_URI, ForumsURL, JOBFORUMS_ID)));
        }

        public void ShowThread(String id)
        {
            HtmlPage.Window.Navigate(new Uri(String.Format(SHOW_THREAD_URI, ForumsURL, id)));
        }

        public void GetFeed()
        {
            WebClient wc = new WebClient();
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(getFeed_DownloadStringCompleted);
            wc.DownloadStringAsync(new Uri(String.Format(URI, JOBFORUMS_ID), UriKind.RelativeOrAbsolute));
        }

        private void getFeed_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                System.Diagnostics.Debug.WriteLine(e.Error.Message);
                return;
            }

            XDocument doc = XDocument.Parse(e.Result);
            // System.Diagnostics.Debug.WriteLine(doc.ToString(SaveOptions.OmitDuplicateNamespaces));

            ForumsURL = doc.Element("source").Element("url").Value;
            ThreadList = from item in doc.Descendants("thread")
                         select new Thread()
                         {
                             Title = item.Element("title").Value,
                             Date = buildDate((item.Element("date").Value as String), (item.Element("time").Value as String)),
                             Id = item.Attribute("id").Value,
                             Author = item.Element("author").Value
                         };
            Initialized = true;

        }

        #endregion

        #region Private Methods

        //NotifyPropertyChanged will raise the PropertyChanged event passing the
        //source property that is being updated.
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }         

        // quick and dirty		
        private DateTime buildDate(String dayMonthYear, String hourMinute)
        {
            DateTime date;
            Array dayMonthYearArray = dayMonthYear.Split(new Char[] { '-' });
            int day = int.Parse(dayMonthYearArray.GetValue(0) as String);
            int month = int.Parse(dayMonthYearArray.GetValue(1) as String);
            int year = int.Parse(dayMonthYearArray.GetValue(2) as String);
            Array hourMinuteArray = hourMinute.Split(new Char[] { ':' });
            int hour = int.Parse(hourMinuteArray.GetValue(0) as String);
            int minute = int.Parse(hourMinuteArray.GetValue(1) as String);
            date = new DateTime(year, month, day, hour, minute, 0);
            return date;
        }

        #endregion

    }
}
